% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic-class.R
\name{plot.nma_dic}
\alias{plot.nma_dic}
\title{Plots of model fit diagnostics}
\usage{
\method{plot}{nma_dic}(
  x,
  y,
  ...,
  show_uncertainty = TRUE,
  stat = "pointinterval",
  orientation = c("vertical", "horizontal", "x", "y")
)
}
\arguments{
\item{x}{A \code{nma_dic} object}

\item{y}{(Optional) A second \code{nma_dic} object, to produce "dev-dev" plots for
model comparison.}

\item{...}{Additional arguments passed on to other methods}

\item{show_uncertainty}{Logical, show uncertainty with a \code{ggdist} plot
stat? Default \code{TRUE}.}

\item{stat}{Character string specifying the \code{ggdist} plot stat to use if
\code{show_uncertainty = TRUE}, default \code{"pointinterval"}. If \code{y} is provided,
currently only \code{"pointinterval"} is supported.}

\item{orientation}{Whether the \code{ggdist} geom is drawn horizontally
(\code{"horizontal"}) or vertically (\code{"vertical"}). Only used for residual
deviance plots, default \code{"vertical"}.}
}
\value{
A \code{ggplot} object.
}
\description{
The \code{plot()} method for \link{nma_dic} objects produced by \code{\link[=dic]{dic()}} produces
several useful diagnostic plots for checking model fit and model comparison.
Further detail on these plots and their interpretation is given by
\insertCite{TSD2;textual}{multinma}.
}
\details{
When a single \code{nma_dic} object is given, a plot of the residual
deviance contribution for each data point is produced. For a good fitting
model, each data point is expected to have a residual deviance of 1; larger
values indicate data points that are fit poorly by the model.

When two \code{nma_dic} objects are given, a "dev-dev" plot comparing the
residual deviance contributions under each model is produced. Data points
with residual deviance contributions lying on the line of equality are fit
equally well under either model. Data points lying below the line of
equality indicate better fit under the second model (\code{y}); conversely, data
points lying above the line of equality indicate better fit under the first
model (\code{x}). A common use case is to compare a standard consistency model
(fitted using \code{\link[=nma]{nma()}} with \code{consistency = "consistency"}) with an unrelated
mean effects (UME) inconsistency model (fitted using \code{\link[=nma]{nma()}} with
\code{consistency = "ume"}), to check for potential inconsistency.

See \insertCite{TSD2;textual}{multinma} for further details.
}
\examples{
## Smoking cessation
# Set up network of smoking cessation data
head(smoking)

smk_net <- set_agd_arm(smoking,
                       study = studyn,
                       trt = trtc,
                       r = r,
                       n = n,
                       trt_ref = "No intervention")

# Print details
smk_net

\donttest{
# Fitting a fixed effect model
smk_fit_FE <- nma(smk_net,
              trt_effects = "fixed",
              prior_intercept = normal(scale = 100),
              prior_trt = normal(scale = 100))

smk_fit_FE
}

\donttest{
# Fitting a random effects model
smk_fit_RE <- nma(smk_net,
                  trt_effects = "random",
                  prior_intercept = normal(scale = 100),
                  prior_trt = normal(scale = 100),
                  prior_het = normal(scale = 5))

smk_fit_RE
}

\donttest{
# Compare DIC of FE and RE models
(smk_dic_FE <- dic(smk_fit_FE))
(smk_dic_RE <- dic(smk_fit_RE))   # substantially better fit

# Plot residual deviance contributions under RE model
plot(smk_dic_RE)

# Changing the plot stat used
plot(smk_dic_RE, stat = "interval", orientation = "horizontal")

# Further customisation is possible using ggplot commands
# For example, highlighting data points with residual deviance above a certain threshold
plot(smk_dic_RE) +
  ggplot2::aes(colour = ifelse(..y.. > 1.5, "darkorange", "black")) +
  ggplot2::scale_colour_identity()

# Or by posterior probability, for example here a central probability of 0.6
# corresponds to a lower tail probability of (1 - 0.6)/2 = 0.2
plot(smk_dic_RE, .width = c(0.6, 0.95)) +
  ggplot2::aes(colour = ifelse(..ymin.. > 1, "darkorange", "black")) +
  ggplot2::scale_colour_identity()

# Check for inconsistency using UME model
}
\donttest{
# Fitting an unrelated mean effects (inconsistency) model
smk_fit_RE_UME <- nma(smk_net,
                      consistency = "ume",
                      trt_effects = "random",
                      prior_intercept = normal(scale = 100),
                      prior_trt = normal(scale = 100),
                      prior_het = normal(scale = 5))

smk_fit_RE_UME
}

\donttest{
# Compare DIC
smk_dic_RE
(smk_dic_RE_UME <- dic(smk_fit_RE_UME))  # no difference in fit

# Compare residual deviance contributions with a "dev-dev" plot
plot(smk_dic_RE, smk_dic_RE_UME)

# By default the dev-dev plot can be a little cluttered
# Hiding the credible intervals
plot(smk_dic_RE, smk_dic_RE_UME, show_uncertainty = FALSE)

# Changing transparency
plot(smk_dic_RE, smk_dic_RE_UME, point_alpha = 0.5, interval_alpha = 0.1)
}
}
\references{
\insertAllCited{}
}
